import { extractArrayInfo, formatType, INTERNAL } from './common';

// A Single Property
export class Property {
    readonly offset: number;
    readonly #type: string;
    readonly #name: string;
    readonly #self: string;

    // Constructor
    constructor(offset: number, type: string, name: string, self: string) {
        this.offset = offset;
        this.#type = type;
        this.#name = name;
        this.#self = self;
    }

    // Getters
    parts() {
        const info = extractArrayInfo(this.#name);
        return {
            type: formatType(this.#type),
            name: info.name,
            arrayInfo: info.arrayInfo
        };
    }
    name() {
        return this.parts().name;
    }
    #fullName(separator: string) {
        return this.#self + separator + this.name();
    }
    fullName() {
        return this.#fullName('_');
    }
    prettyName() {
        return this.#fullName('::');
    }
    rawType() {
        return this.#type;
    }
}

// A Single Static Property
export class StaticProperty extends Property {
    type(pointer: boolean) {
        const parts = this.parts();
        return `${INTERNAL}make_${pointer ? 'pointer' : 'reference'}<${parts.type.trim()}${parts.arrayInfo}>`;
    }
    // Reference
    referenceDefinition(addSelf: boolean) {
        return `${this.type(false)} ${addSelf ? this.prettyName() : this.name()}`;
    }
}